#include "testApp.h"
#include <algorithm>

//--------------------------------------------------------------
void testApp::setup(){	 

	ofBackground(0,0,0);
	ofSetFrameRate(30);
	ofEnableAlphaBlending();
	
	deckState = 0;
	
	nextCard = 0;
	nextSuit = 0;
	
	currentCard = 0;
	lastTime = ofGetElapsedTimeMillis();
	elapsedTime = 0;
	
	rotateCard = false;
	removeCard = false;
	
}

//--------------------------------------------------------------
void testApp::update(){

	int currentTime = ofGetElapsedTimeMillis();
	elapsedTime += currentTime - lastTime;
	lastTime = currentTime;
	
	if(cards.size()>1){
	
		if(elapsedTime > cards[currentCard]->noteDuration){
		
			if(cards[currentCard]->isLooping) cards[currentCard]->stop();
			cards[currentCard]->active = false;
			
			currentCard++;
			
			if(currentCard>=cards.size()){
			
				currentCard = 0;
			}
			
			
			//cards[currentCard]->snd.setVolume(0.125);
			cards[currentCard]->play();
			cards[currentCard]->active = true;
			
			//printf("current card %i\n", currentCard);
			
			elapsedTime = 0;
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){

	int maxDuration = 4000;

	for(int i=0; i<cards.size(); i++){
	
		cards[i]->draw();
		
	}
	
	ofSetColor(255, 255, 255);
	
	// show next card
	string cards[] = { "A", "2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K" };
	string nCard = "next card (~ to +): "+cards[nextCard];
	ofDrawBitmapString(nCard, 20, 25);
	
	// show next suit
	string suits[] = { "H", "C", "D", "S" };
	string nSuit = "next suit (Q/W/E/R): "+suits[nextSuit];
	ofDrawBitmapString(nSuit, 20, 50);
	
	// show rotate mode
	string fpsStr = "rotate card (z): "+ofToString(rotateCard);
	ofDrawBitmapString(fpsStr, 20, 75);
	
	// show remove mode
	string remCard = "remove card (c): "+ofToString(removeCard);
	ofDrawBitmapString(remCard, 20, 100);

}

//--------------------------------------------------------------
bool sortFunc(ofPlayingCard * cardA, ofPlayingCard * cardB){

	return cardA->x < cardB->x;
}

//--------------------------------------------------------------
void testApp::sortCards(){

	std::sort(cards.begin(), cards.end(), sortFunc);
	
	calculateDistanceBetweenCards();
}

//--------------------------------------------------------------
void testApp::calculateDistanceBetweenCards(){

	int cardCount = cards.size();

	if(cardCount>1){
	
		for(int i=1; i<cardCount; i++){
		
			int dist = cards[i]->x - cards[i-1]->x;
			
			cards[i-1]->noteDuration = dist * 2;
		}
		
		int lastDistance = cards[0]->x + ofGetWidth() - cards[cardCount-1]->x;
		cards[cardCount-1]->noteDuration = lastDistance * 2;
		
	}
}

//--------------------------------------------------------------
void testApp::placeNewCard(int x, int y){

	// create a new card
	ofPlayingCard * newCard = new ofPlayingCard(x, y, nextCard, nextSuit);
	
	// sound file location
	string suiteFiles[] = { "waveforms/ei_sine.aiff", "samples/bass/Bass-002.aif", "samples/guitar/Guitar-001.aif", "waveforms/ei_noise.aiff" }; 
	bool loopFile[] = { true, false, false, true };
	
	// card speeds (jack/queen/king are octaves)
	float fileSpeed[] = { 1, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2.0, 3.0, 4.0 };
	
	if(nextSuit == 0){
	
		// if hearts, load specific sound
		// keep speed at 1
	
		string heartCards[] = {
	
			"samples/drums/h16_csh.aif",
			"samples/drums/h16_hat.aif",
			"samples/drums/h16_kik.aif",
			"samples/drums/h16_ped.aif",
			"samples/drums/h16_rd2.aif",
			"samples/drums/h16_rid.aif",
			"samples/drums/h16_snr.aif",
			"samples/drums/o_clap.aif",
			"samples/drums/o_crsh.aif",
			"samples/drums/o_kick.aif",
			"samples/drums/o_ride.aif",
			"samples/drums/o_rims.aif",
			"samples/drums/o_snare.aif"
		
		};
	
		// load the sound
		newCard->loadSound(heartCards[nextCard], false);
		newCard->snd.setVolume(1.0-float(y)/ofGetHeight());
		newCard->snd.setSpeed(1.0);
	
	} else {
	
		// load the sound
		newCard->loadSound(suiteFiles[nextSuit], loopFile[nextSuit]);
		newCard->snd.setVolume(1.0-float(y)/ofGetHeight());
		newCard->snd.setSpeed(fileSpeed[nextCard]);
	}

	// add to vector
	cards.push_back(newCard);
	
	// play the first card (but only if there's just one on the screen)
	if(cards.size() == 1){
		
		cards[0]->active = true;
		cards[0]->snd.play();
		
		lastTime = ofGetElapsedTimeMillis();
	}
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	// table for all key presses for card
	int cardArray[] = { 96, 49, 50, 51, 52, 53, 54, 55, 56, 57, 48, 45, 61 };
	
	// table for all key presses for suit
	int suitArray[] = { 113, 119, 101, 114 };
	
	for(int i=0; i<13; i++){
	
		if(cardArray[i] == key){
		
			nextCard = i;
		}
	}
	
	for(int i=0; i<4; i++){
	
		if(suitArray[i] == key){
		
			nextSuit = i;
		}
	}
	
	if(key == 122){
	
		// z pressed
	
		rotateCard = !rotateCard;
		removeCard = false;
	
	} else if(key == 99){
	
		// c pressed
	
		removeCard = !removeCard;
		rotateCard = false;
	}
	
	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){ 
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	
	if(deckState == 1){
	
		if(rotateCard){
		
			// calculate x pos of mouse in relation to card
			int cardX = cards[cardSelected]->x;
			
			// rotate card
			cards[cardSelected]->rotation = -1 * (cardX - x);
			
		
		} else {
		
			// set new x and y pos
			cards[cardSelected]->x = x;
			cards[cardSelected]->y = y;
			
			// change speed based on pos
			cards[cardSelected]->snd.setVolume(1.0-float(y)/ofGetHeight());
		}
	}

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

	// search through all cards looking for mouse interaction
	
	bool ifClicked = false; 
	
	for(int i=0; i<cards.size(); i++){
	
		bool test = cards[i]->ifCardClicked(x, y);
		
		if(test) { cardSelected = i; ifClicked = true; }
	}
	
	if(ifClicked) {
	
		// if true, enable dragging
	
		deckState = 1;
		
		if(removeCard && cards.size() >= 1){
		
			// remove a card
			cards[cardSelected]->stop();
			cards.erase(cards.begin()+cardSelected);
			
			// disabe card removal
			removeCard = !removeCard;
		}
		
	} else {
	
		// otherwise, place a new card

		deckState = 0;
		placeNewCard(x, y);
		sortCards();
		
		// now card moving is enabled (dragging last card)
		deckState = 1;
		cardSelected = cards.size()-1;
	}
	
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(){

	// sort card positions
	if(deckState==1) sortCards();

	deckState = 0;
	
	// disable rotate mode
	if(rotateCard) rotateCard = !rotateCard;
}
